"""
Defines the version for the compatible MinGW-w64 compiler from https://winlibs.com/.

(-) Release scripts might modify this automatically...
(-) do not add anything but the version number of the wasi-sdk compiler here!

"""

__version__ = "13.2.0posix-18.1.5-11.0.1-ucrt-r8"
