"""
Defines the version for the compatible rust compiler.

(-) Release scripts might modify this automatically...
(-) do not add anything but the version number of the rust compiler here!

"""

__version__ = "1.79.0"
