// Constants for the Mock IC

#pragma once

#include <string>

constexpr std::string_view MOCKIC_CANISTER_SELF = "5ugrv-zqaaa-aaaag-acfna-cai";
constexpr __uint128_t MOCKIC_INITIAL_CYCLES_BALANCE = 3000000000000;
constexpr std::string_view MOCKIC_CONTROLLER =
    "expmt-gtxsw-inftj-ttabj-qhp5s-nozup-n3bbo-k7zvn-dg4he-knac3-lae";